package tech.riemann.ims.controller.platform.acl;

import java.util.Arrays;
import java.util.List;

import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import club.zhcs.lina.starter.exception.BizException;
import club.zhcs.lina.utils.enums.Codebook;
import club.zhcs.lina.utils.enums.ICodeBook;
import club.zhcs.lina.utils.tree.Tree;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import tech.riemann.ims.entity.acl.Permission;
import tech.riemann.ims.service.acl.IPermissionService;

/**
 * <p>
 * 权限 前端控制器
 * </p>
 *
 * @author Kerbores(kerbores@gmail.com)
 *
 * @since 2024-07-22 13:56:54
 */
@RestController
@Tag(name = "Permission", description = "权限")
@RequiredArgsConstructor
public class PermissionController {

    private final IPermissionService permissionService;

    @GetMapping("permission/types")
    @Operation(summary = "权限类型")
    public List<Codebook> types() {
        return Arrays.stream(Permission.Type.values())
                     .map(ICodeBook::build)
                     .toList();
    }

    @GetMapping("permissions/tree")
    @Operation(summary = "权限树")
    public List<Tree<String>> permissionTree(
                                             @Parameter(description = "强制拉取") @RequestParam(required = false) Boolean force) {
        return permissionService.permissionTree(force);
    }

    @GetMapping("permissions")
    @Operation(summary = "权限列表")
    public List<Permission> permissions(
                                        @Parameter(description = "强制拉取") @RequestParam(required = false) Boolean force) {
        return permissionService.permissions(force);
    }

    @GetMapping("permission/{id}")
    @Operation(summary = "权限详情")
    public Permission detail(@Parameter(description = "权限id", required = true) @PathVariable long id) {
        return permissionService.getById(id);
    }

    @PostMapping("permission")
    @Operation(summary = "增加权限")
    public Permission add(@Validated @Parameter(description = "权限") @RequestBody Permission permission) {
        if (permissionService.save(permission)) {
            return permission;
        } else {
            throw BizException.create("保存权限失败!");
        }
    }

    @PatchMapping("permission")
    @Operation(summary = "更新权限")
    public Permission update(@Validated @Parameter(description = "权限") @RequestBody Permission permission) {
        if (permissionService.update(permission, Wrappers.<Permission> lambdaUpdate().eq(Permission::getId, permission.getId()))) {
            return permission;
        } else {
            throw BizException.create("更新权限失败!");
        }
    }

    /**
     * 删除权限
     * 
     * @param id
     *            权限id
     * @return 是否删除成功
     */
    @DeleteMapping("permission/{key}")
    @Operation(summary = "删除权限")
    public void deletePermission(@Parameter(description = "权限key", required = true) @PathVariable String key) {
        if (!permissionService.remove(Wrappers.<Permission> lambdaQuery().eq(Permission::getKey, key))) {
            throw BizException.create("删除权限失败!");
        }
    }

    @PostMapping("batch-init-permissions")
    @Operation(summary = "批量新增权限")
    @ResponseStatus(value = HttpStatus.OK)
    public void batchInitPermissions(
                                     @RequestBody List<Permission> permissions) {
        permissionService.batchAddPermissions(permissions);
    }

    @PatchMapping("batch-sync-permissions")
    @Operation(summary = "增量新增权限")
    @ResponseStatus(value = HttpStatus.OK)
    public void batchSyncPermissions(@RequestBody List<Permission> permissions) {
        permissionService.updatePermissions(permissions);
    }

}
